<?xml version="1.0" encoding="UTF-8"?>
<!--
########################################################################
# metaf-add.xsl
#
# copyright (c) 2006-2021 metaf2xml
#
# This file is part of metaf2xml.
#
# metaf2xml is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# metaf2xml is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with metaf2xml.  If not, see <https://www.gnu.org/licenses/>.
########################################################################
-->

<!--

example transformation to:
- change speed to km/h, distance to m, temperature to °C, pressure to hPa
- add station info as node <info> below m:obsStationId|m:callSign|m:buoyId|m:aircraftId

-->

<!DOCTYPE xsl:stylesheet [
<!ENTITY xslns 'http://www.w3.org/1999/XSL/Transform'>
<!ENTITY m2xns 'http://metaf2xml.sourceforge.net/2.9'>
<!ENTITY nl    '<text xmlns="&xslns;">&#10;    </text>'>
]>

<xsl:stylesheet
    xmlns:xsl = "&xslns;"
    xmlns:adddata = "urn:data-section"
    xmlns:m = "&m2xns;"
    xmlns   = "&m2xns;"
    exclude-result-prefixes = "adddata m"
    version = "1.0">

 <xsl:output
    method = "xml"
    encoding = "UTF-8"
    indent = "yes"
 />

 <xsl:variable name="NM2KM" select="1.852"/>
 <xsl:variable name="FT2M" select="0.3048"/>
 <xsl:variable name="SM2M" select="1609.3412196"/> <!-- statute mile! -->
 <xsl:variable name="METER_PER_SEC2KT" select="3.6 div $NM2KM"/>
 <xsl:variable name="INHG2HPA" select="33.86388640341"/>

 <xsl:template match="/data">
  <xsl:element name="data" namespace="">
   <xsl:apply-templates/>
  </xsl:element>
 </xsl:template>

 <xsl:template match="m:metar/m:obsStationId|m:taf/m:obsStationId">
  <obsStationId s="{@s}">
   <xsl:if test="boolean(document('')//adddata:obsStationInfo/station[@icao=current()/m:id/@v])">
    &nl;
    <info xmlns="">
     <xsl:apply-templates select="document('')//adddata:obsStationInfo/station[@icao=current()/m:id/@v]"/>
    </info>
   </xsl:if>
   <xsl:apply-templates/>
  </obsStationId>
 </xsl:template>

 <xsl:template match="m:synop/m:obsStationId">
  <obsStationId s="{@s}">
   <xsl:if test="boolean(document('')//adddata:obsStationInfo/station[@wmo=current()/m:id/@v])">
    &nl;
    <info xmlns="">
     <xsl:apply-templates select="document('')//adddata:obsStationInfo/station[@wmo=current()/m:id/@v]"/>
    </info>
   </xsl:if>
   <xsl:apply-templates/>
  </obsStationId>
 </xsl:template>

 <xsl:template match="m:callSign[../m:obsStationType/m:stationType/@v = 'BBXX']">
  <callSign s="{@s}">
   <xsl:if test="boolean(document('')//adddata:obsStationInfo/station[@ship=current()/m:id/@v])">
    &nl;
    <info xmlns="">
     <xsl:apply-templates select="document('')//adddata:obsStationInfo/station[@ship=current()/m:id/@v]"/>
    </info>
   </xsl:if>
   <xsl:apply-templates/>
  </callSign>
 </xsl:template>

 <xsl:template match="m:callSign[../m:obsStationType/m:stationType/@v = 'OOXX']">
  <callSign s="{@s}">
   <xsl:if test="boolean(document('')//adddata:obsStationInfo/station[@mobil=current()/m:id/@v])">
    &nl;
    <info xmlns="">
     <xsl:apply-templates select="document('')//adddata:obsStationInfo/station[@mobil=current()/m:id/@v]"/>
    </info>
   </xsl:if>
   <xsl:apply-templates/>
  </callSign>
 </xsl:template>

 <xsl:template match="m:buoyId">
  <buoyId s="{@s}">
   <xsl:if test="boolean(document('')//adddata:obsStationInfo/station[@buoy=current()/m:id/@v])">
    &nl;
    <info xmlns="">
     <xsl:apply-templates select="document('')//adddata:obsStationInfo/station[@buoy=current()/m:id/@v]"/>
    </info>
   </xsl:if>
   <xsl:apply-templates/>
  </buoyId>
 </xsl:template>

 <xsl:template match="m:aircraftId">
  <aircraftId s="{@s}">
   <xsl:if test="boolean(document('')//adddata:obsStationInfo/station[@ac=current()/m:id/@v])">
    &nl;
    <info xmlns="">
     <xsl:apply-templates select="document('')//adddata:obsStationInfo/station[@ac=current()/m:id/@v]"/>
    </info>
   </xsl:if>
   <xsl:apply-templates/>
  </aircraftId>
 </xsl:template>

 <xsl:template name="print_v">
  <xsl:param name="v"/>
  <xsl:param name="u"/>
  <xsl:element name="{name()}">
   <xsl:attribute name="v"><xsl:value-of select="$v"/></xsl:attribute>
   <xsl:attribute name="u"><xsl:value-of select="$u"/></xsl:attribute>
   <xsl:if test="boolean(@q)">
    <xsl:attribute name="q"><xsl:value-of select="@q"/></xsl:attribute>
   </xsl:if>
  </xsl:element>
 </xsl:template>

 <xsl:template match="m:speed|m:gustSpeed">
  <xsl:variable name="v">
   <xsl:choose>
    <xsl:when test="@u = 'MPS'">
     <xsl:value-of select="@v * 3.6"/>
    </xsl:when>
    <xsl:when test="@u = 'KT'">
     <xsl:value-of select="@v * $NM2KM"/>
    </xsl:when>
    <xsl:when test="@u = 'CMSEC'">
     <xsl:value-of select="@v * 3.6 div 100"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="@v"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>
  <xsl:call-template name="print_v">
   <xsl:with-param name="v" select="$v"/>
   <xsl:with-param name="u" select="'KMH'"/>
  </xsl:call-template>
 </xsl:template>

 <xsl:template match="m:temp">
  <xsl:variable name="v">
   <xsl:choose>
    <xsl:when test="@u = 'F'">
     <xsl:value-of select="(@v - 32) div 1.8"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="@v"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>
  <xsl:call-template name="print_v">
   <xsl:with-param name="v" select="$v"/>
   <xsl:with-param name="u" select="'C'"/>
  </xsl:call-template>
 </xsl:template>

 <xsl:template match="m:distance">
  <xsl:variable name="v">
   <xsl:choose>
    <xsl:when test="@u = 'KM'">
     <xsl:value-of select="@v * 1000"/>
    </xsl:when>
    <xsl:when test="@u = 'SM'">
     <xsl:value-of select="@v * $SM2M"/>
    </xsl:when>
    <xsl:when test="@u = 'FT'">
     <xsl:value-of select="@v * $FT2M"/>
    </xsl:when>
    <xsl:when test="@u = 'FL'">
     <xsl:value-of select="@v * $FT2M * 100"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="@v"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>
  <xsl:call-template name="print_v">
   <xsl:with-param name="v" select="$v"/>
   <xsl:with-param name="u" select="'M'"/>
  </xsl:call-template>
 </xsl:template>

 <xsl:template match="m:pressure">
  <xsl:variable name="v">
   <xsl:choose>
    <xsl:when test="@u = 'kPa'">
     <xsl:value-of select="@v div 10"/>
    </xsl:when>
    <xsl:when test="@u = 'inHg'">
     <xsl:value-of select="@v * $INHG2HPA"/>
    </xsl:when>
    <xsl:when test="@u = 'mmHg'">
     <xsl:value-of select="@v div ($FT2M div 12 * 1000) * $INHG2HPA"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="@v"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>
  <xsl:call-template name="print_v">
   <xsl:with-param name="v" select="$v"/>
   <xsl:with-param name="u" select="'hPa'"/>
  </xsl:call-template>
 </xsl:template>

 <xsl:template match="processing-instruction('xml-stylesheet')">
  <xsl:processing-instruction name="xml-stylesheet">
   <xsl:value-of select="."/>
  </xsl:processing-instruction>
 </xsl:template>

 <xsl:template match="adddata:obsStationInfo/station">
  <xsl:for-each select="@ctry|@loc">
   <xsl:copy/>
  </xsl:for-each>
  <xsl:value-of select="."/>
 </xsl:template>

 <xsl:template match="comment()">
  <xsl:comment><xsl:value-of select="."/></xsl:comment>
 </xsl:template>

 <xsl:template match="*">
  <xsl:element name="{name()}">
   <xsl:for-each select="@*">
    <xsl:copy/>
   </xsl:for-each>
   <xsl:apply-templates/>
  </xsl:element>
 </xsl:template>

 <adddata:obsStationInfo xmlns="">
  <station wmo="83746" icao="SBGL" ctry="BR" loc="2249S04315W0006">Aeroporto Internacional do Rio de Janeiro/Galeão</station>
  <station wmo="74486" icao="KJFK" ctry="US-NY" loc="4039N07347W0007">Kennedy International Airport, New York</station>
  <station wmo="47671" icao="RJTT" ctry="JP" loc="3533N13947E0008">Tokyo International</station>
  <station wmo="47662" icao="RJTD" ctry="JP" loc="3541N13946E0036">Metropolitan Area Heliport Tokyo</station>
  <station ship="A8XY2" loc="3718N07154W0000">ship A8XY2</station>
  <station mobil="AAWFD" loc="7748S16654E0014">mobil AAWFD</station>
  <station buoy="31533" loc="294937S0452241W0000">buoy 31533</station>
  <station buoy="44769" loc="354906N0693130W0000">buoy 44769</station>
  <station buoy="52801" loc="332440N1414941E0000">buoy 52801</station>
  <station ac="NZL020" loc="4311S17205E4876">aircraft NZL020</station>
 </adddata:obsStationInfo>

</xsl:stylesheet>
